c######## main ######################################################
c
      implicit real*8(a-h,o-z)
      include 'maxdim.h'
      save
      dimension x(maxatom),y(maxatom),z(maxatom),vsip(2)
      common /parsys/ vsip,upenalty,cem,unite,unitd
      open(7,file='al.inp')
      open(9,file='al.out')
c      open(13,file='wydruk')
      read(7,*)natom,icase,ipr
      do 1 i=1,natom
    1 read(7,*) x(i),y(i),z(i)
c      write(9,*) natom,icase
      do 5 i=1,natom
      x(i)=x(i)*unitd
      y(i)=y(i)*unitd
      z(i)=z(i)*unitd
c      write(9,3) x(i)/unitd,y(i)/unitd,z(i)/unitd
    5 continue
c      ionpar=0
      call surf(v,x,y,z,natom,icase,ipr)
c      write(9,3)v/unite
    3 format(5d16.6)
      stop
      end
c
c######################################################################
c
      SUBROUTINE SURF(vv,x,y,z,natom,icase,ipr)
c
c  Calculates the potential energy surfaces for Al nanoparticles 
c  using many body tight binding theory (MBTB).
c  The Hamiltonian matrix elements are parametrized in 6 different 
c  ways described in Ref.:
c  G.Staszewska, P.Staszewski, N. E. Schultz and D. G. Truhlar, 
c  Many body tight-binding model for aluminum nanoparticles. 
c  Phys. Rev. B  submitted.
c  1 MBTB-S  - many-body tight-binding based on screening,
c  2 MBTB-CN - many-body tight-binding based on coordination number, 
c  3 MBTB-BA - Many-body tight-binding based on bond-angle correction,
c    and three versions of pair-wise tight binding:
c  4 TB-OWH  - optimized Wolfsberg-Helmholz approximation
c  5 TB-EWH  - extended Wolfsberg-Helmholz approximation
c  6 TB-WH   - Wolfsberg-Helmholz approximation.
c
c  calculations are performed in atomic units
c
c  vv    --- energy of the system
c  x,y,z --- one-dimensional arrays of the Cartesian coordinates 
c            of the system.
c  natom --- number of atoms in the system
c  icase --- parameter teling which model is to be used
c          1--MBTB-S
c          2--MBTB-CN
c          3--MBTB-BA
c          4--OWH
c          5--EWH
c          6--WH
c  ipr --- parameter teling wether print information about HOMO, LUMO
c          eigenvalues, occupation number and partial charge.
c          =0, do not print
c          ne.0 print
c                                                         
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      character*75 space, hyphen
      character*75  tbcase
      parameter (space= "                                  ")
      parameter (hyphen="-----------------------------------------------
     *---------------------------")
      dimension x(maxatom),y(maxatom),z(maxatom),r(maxatom,maxatom)
      dimension par(npar),vsip(2),tbcase(6)
      dimension hamilt(maxdim,maxdim),hamil(norbit,norbit),q(maxdim)
      dimension ap(maxdim*(maxdim+1)/2),work(3*maxdim),nocci(maxdim)
      dimension evec(maxdim,maxdim),eva(maxdim),nocc(maxdim)
      common /parsys/ vsip,upenalty,cem,unite,unitd
      common/param/par
c     
      tbcase(1)='   Many-body tight-binding based on screening (MBTB-S)' 
      tbcase(2)='   Many-body tight-binding based on coordination number 
     * (MBTB-CN) ' 
      tbcase(3)=' Many-body tight-binding based on bond-angle correction 
     * (MBTB-BA) ' 
      tbcase(4)='Tight-binding, Optimized Wolfsberg-Helmholz approximati
     *on  (TB-OWH) ' 
      tbcase(5)='Tight-binding, Extended Wolfsberg-Helmholz approximatio
     *n (TB-EWH) ' 
      tbcase(6)='Tight-binding, Wolfsberg-Helmholz approximation (TB-WH)
     *  ' 
      write(9,*)  
               write(9,*)tbcase(icase)
      write(9,*) 
      write(9,*)"   Number of atoms in the cluster =",natom
      write(9,*)
      write(9,*)'                        Geometry of the system'
      write(9,*)
      write(9,*)'            in bohrs                      in angstroms'
      un=0.52917697
             do i=1,natom
      write(9,107)x(i),y(i),z(i),x(i)*un,y(i)*un,z(i)*un
             enddo
  107 format(3f10.5,5x,3f10.5)   
      write(9,*) 
      nElect=3*natom
      Tzero=natom*(2d0*vsip(1)+vsip(2)+upenalty)
c
c   calculate distances between atoms, r
c
c      write(9,*)natom
      do 1 i=1,natom
      r(i,i)=0d0
      do 1,j=i+1,natom
      dx=x(i)-x(j)
      dy=y(i)-y(j)
      dz=z(i)-z(j)
      r(i,j)=dsqrt(dx*dx+dy*dy+dz*dz)
    1 r(j,i)=r(i,j)
c
c  take parameters for the model
c
      call case(icase)
c
c  calculate auxialiary arrays for many-body terms
c
      if(icase.le.3)call envtab(r,natom,icase)
c
c  calculate hamiltonian matrix
c
      ndim=natom*norbit
      do 5 i=1,ndim
      do 5 j=1,ndim
    5 hamilt(i,j)=0d0
      do 6 i=1,natom
c
c  calculate diagonal elements
c
      call vsips(r,vs,vp,natom,i,icase)
      i1=norbit*(i-1)+1
      i2=i1+1
      i3=i2+1
      i4=i3+1
      hamilt(i1,i1)=vs
      hamilt(i2,i2)=vp
      hamilt(i3,i3)=vp
      hamilt(i4,i4)=vp
    6 continue
      eerep=0d0
      nrdim=0
      do 30 i=1,natom
      ncdim=0
      do 31 j=1,natom
      if(i.eq.j) go to 31
c
c  calculate nondiagonal elements
c
      call pot_al (x,y,z,r,hamil,repul,natom,i,j,icase)
      do 34 ii=1,norbit
      do 35 jj=1,norbit
      hamilt(nrdim+ii,ncdim+jj)=hamil(ii,jj)
 35   continue
 34   continue
      eerep=eerep+repul
 31   ncdim=ncdim+norbit
      nrdim=nrdim+norbit
 30   continue
c
c      do 200 i=1,ndim
c  200 write(9,201)(hamilt(i,j),j=1,ndim)
  201 format(8d14.6)
c
      eerep=eerep*0.5d0
c 
c Set up the uppermatrix to do eigenvalue calculations.
c
      do 40 i =1,ndim 
      do 41 j=i,ndim
      ap(i+(j-1)*j/2)=hamilt(i,j)
 41   continue 
 40   continue

c      do 501 i=1,ndim
c  501 write(9,401)(hamilt(i,j),j=1,ndim)
c
c  calculate eigenvalues and eigenvectors
c
      call dspev( 'v','u',ndim,ap,eva,evec,maxdim,work,info ) 
c
  401 format(8d16.8)
c
c   fill orbitals
c
      call filloc(ndim,nElect,eva,etb,nocc)
c
c   calculate multiplicity
c   it is assumed that spin in single occupied states is up
c
      nspin=0
      do 738 i=1,ndim
      if(nocc(i).eq.1)nspin=nspin+1
  738 continue
      multi=nspin+1

  334 format(8d11.4)
c#######################################################################
c
c Calculate Mulliken net AO Population for each atomic orbital
c Please refer to J. P. Lowe, Quantum Chemistry, p336 (Second edition,
c Acedemic press, 1993).  Note: Mulliken overlap population is zero since
c the overlap matrix here is assumed as unit matrix.
c
c
      if(ipr.eq.0)go to 335 
c#######################################################################
      do j = 1, ndim
        q(j)=0.0
        do i = 1, ndim
          q(j) = q(j) +nocc(i)*evec(j,i)*evec(j,i)
        enddo
      enddo
c
c Calculate HOMO orbitals
c
      write (9,*) "                   Information about HOMO and LUMO"
      write (9,*) hyphen
      write (9,97) " MO No.", "Eigenvalue","description"
      write(9,*)'         hartree    eV   '
      write (9,*) hyphen
      do i = 1, ndim
        if (nocc(i).eq.2) then
          if (nocc(i+1).eq.1) then
       write(9,98) i,eva(i),eva(i)*27.21161,' This is highest doubly occ 
     *upied orbital.'
            write(9,98) i+1,eva(i+1),eva(i+1)*27.21161,' This is lowest 
     *singly occupied.'
          else if (nocc(i+1).eq.0) then
          write(9,98) i,eva(i),eva(i)*27.21161,' This is highest doubly 
     *occupied orbital.'
            write(9,*) "There is no singly-occupied orbital."
            write(9,98) i,eva(i),eva(i)*27.21161,' This is HOMO.'
            potion=-eva(i)
            if ((i+1) .gt. ndim) then
              write (9,*) 'There is no LUMO'
            else
             write(9,98) i+1,eva(i+1),eva(i+1)*27.21161,' This is LUMO.'
            endif
            goto 44
          endif
        else if (nocc(i).eq.1) then
          if (nocc(i+1).eq.0) then
            write(9,98) i,eva(i),eva(i)*27.21161,' This is HOMO.'
              potion=-eva(i)
            if ((i+1) .gt. ndim) then
              write (9,*) 'There is no LUMO'
            else
             write(9,98) i+1,eva(i+1),eva(i+1)*27.21161,' This is LUMO.'
            endif
            goto 44
          endif
        endif
      enddo
c
c Print Out eva, occ, q
c
 44   continue
      write (9,*) hyphen
      write (9,*) space
c
c      do i=0,ndim/8
c        if (i.lt.ndim/8) then
         write (9,*) "Eigenvalues (E), occupation number(n), and partial
     * charge (q)"
c          write (9,*) " followed by Hamiltonian matrix H (in a.u.)"
          write (9,*) hyphen

          write(9,*)'      E(hartree)       E(eV)        n            q'     

              do id=1,ndim
          write (9,108) eva(id),eva(id)*27.21161,nocc(id),q(id)
              enddo
  108 format(5x,f10.5,5x,f10.4,5x,i3,5x,f10.5)
          write (9,*) hyphen
c          write (9,*) hyphen
      write (9,*) space
  335 continue
c#######################################################################
c
c  calculate total energy
c
      VV=etb+eerep-(Tzero)
c
c
      write (9,102) "Energy of the system = ",VV, " hartrees = ",
     &VV*27.21161, "eV = ", VV*627.50, " kcal/mol."
c      write (9,*) hyphen
      write (9,*) space
      write(9,*)"Multiplicity of the system  =",multi
      write(9,*)space


c                   write(9,*) ' v,      etb,     eerep,     Tzero'
c      write(9,105)vv,etb,eerep,Tzero
c   calculate ionization energy
c
c      if(ionpar.ne.0) go to 502
      nElecti=nElect-1
      call filloc(ndim,nElecti,eva,etbi,nocci)
      potion=etbi-etb
c  502 continue
c      write(9,*)
      write(9,503) potion/unite
  503 format(' Ionization energy = ',f10.5,' eV')
      write(9,*)
      write (9,*) hyphen
      write (9,*) space
  502 continue
c
c  end ionization energy

  105 format(6d16.8)
  106 format(8f10.5)
c  105 format(' v,  etb, eerep,  Tzero', 6d14.5)

 95   format (T5,A,T10,3f10.5)
 97   format (T1,A,T13,A,T39,A)
 98   format (T1,I3,f11.4,f12.4,T31,A)
 99   format(T1,A8,8(f9.4))
 100  format(T1,A3,I3,8(f9.4))
 101  format(T3,A,T10,5(f10.5))
 102  format(T2,A,T25,f10.5,T35,A,T47,f10.3,T58,A,T63,f10.3,T73,A)
 103  format(T1,A8,8(I9))
c
      return
      END 
c
      subroutine fbaij(b1,b2,b3,b4,r,tabv,fij,i,j)
c b1,b2,b3,b4 ---environmental parameters contained in the
c                bond-angle function 
c r --- two-dimensional array containing distances between all atoms
c tabv- one dimensional array containing function stored in envtab
c fij -- bond-angle function                                  
c i,j   --- indices of the two considered atoms

c
c  calculate bond-angle term fij changing interaction
c  between atoms i and j
c
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension r(maxatom,maxatom),tabv(maxatom)
      rij=r(i,j)
      s3=dexp(-b2*rij**b3)
      s=(tabv(i)-s3)*(tabv(j)-s3)*b1
      ep=dexp(-2*s)
c      em=1d0/ep
      fij=1d0-((1d0-ep)/(1d0+ep))**b4
c      write(19,4)"fcorr", i,j,fij
    4 format(A5,2i2,4d13.6)
      return
      end
c
      subroutine fsij(b1,b2,b3,natom,r,fij,i,j)
c b1,b2,b3 ---environmental parameters contained in the
c                screening function 
c natom  --- number of atoms in the cluster
c r --- two-dimensional array containing distances between all atoms
c tabv- one dimensional array containing function stored in envtab
c fij -- screening function                                  
c i,j   --- indices of the two considered atoms

c
c  calculate screening term fij changing interaction
c  between atoms i and j
c
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension r(maxatom,maxatom)
      rij=r(i,j)
      s=0d0
      do 1 l=1,natom
      if(l.eq.i.or.l.eq.j) go to 1
      ril=r(i,l)
      rlj=r(l,j)
      s=s+dexp(-b2*((ril+rlj)/rij)**b3)
    1 continue
      s=s*b1
      ep=dexp(-2*s)
c      em=1d0/ep
      fij=1d0-(1d0-ep)/(1d0+ep)
c      write(19,4)"fcorr", i,j,fij
    4 format(A5,2i2,4d13.6)
      return
      end
c
      subroutine fcnij(d,gamma,gzero,ag,bg,r,gcoor,fij,i,j)
c d, gamma, gzero, ag, bg ---environmental parameters contained in the
c              coordination number function 
c r --- two-dimensional array containing distances between all atoms
c tabv- one dimensional array containing function stored in envtab
c fij --coordination number function                                  
c i,j   --- indices of the two considered atoms

c
c  calculate coordination number term fij changing interaction
c  between atoms i and j
c
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension r(maxatom,maxatom),gcoor(maxatom)
      rij=r(i,j)
      wterm=0d0
      if(rij.ge.bg) go to 991
      wterm2=(rij/bg)
      wterm1=ag*(1d0-1d0/(1d0-wterm2))
      wterm=dexp(wterm1)
 991  continue
      cz2=(gcoor(j)-wterm)/gzero
      cz1=(gcoor(i)-wterm)/gzero
      abscz1=dabs(cz1)
      if(abscz1.le.10d-12)cz1=abscz1
      abscz2=dabs(cz2)
      if(abscz2.le.10d-12)cz2=abscz2
      es=1d0/(1d0+cz1**gamma)
      ec=1d0/(1d0+cz2**gamma)
      fij=(1d0+d*(es*ec-1d0))
c      write(19,4)"fcorr", i,j,fij
    4 format(A5,2i2,4d13.6)
      return
      end

c
c
      subroutine vsips(r,vs,vp,natom,iatom,icase)
c
c  calculate valence state ionization energy
c  (diagonal elements of hamiltonian matrix)  
c  r --- two-dimensional array containing distances between all atoms
c  vs,  vp  ---- calculated values of vsip
c  natom --- number of atoms                                         
c  iatom --- ith atom for which the vsip is calculated
c  ivsip --- =1 for s and 2 for p
c
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension r(maxatom,maxatom),vsip(2),tabv(maxatom)
      dimension par(npar)
      dimension csss(maxatom),csps(maxatom),cpps(maxatom),gcoor(maxatom)
      dimension cppp(maxatom),crep(maxatom),cis(maxatom),cip(maxatom) 
      common/envir/csss,csps,cpps,cppp,crep,cis,cip,gcoor
      common/param/par
      common /parsys/ vsip,upenalty,cem,unite,unitd
c 
      vs=vsip(1)
      vp=vsip(2)
      go to (20,30,40,50,50,50),icase
   20 deltacs=par(12)
      deltass=par(13)
      deltacp=par(14)
      deltasp=par(15)
      b1=par(22)
      b2=par(23)
      b3=par(24) 
      ss=0d0
      sp=0d0
      do 21 i=1,natom
      if(i.eq.iatom)go to 21
      ria=r(i,iatom)
      terms=dexp(-deltass*ria)
      termp=dexp(-deltasp*ria)
      call fsij(b1,b2,b3,natom,r,fp,i,iatom)
      ss=ss+terms
      sp=sp+termp*fp
   21 continue
      vs=vs+ss*deltacs
      vp=vp+sp*deltacp
      return
   30 deltacs=par(12)
      deltass=par(13)
      deltacp=par(14)
      deltasp=par(15)
      ds=par(26)
      gammas=par(27)
      dp=par(28)
      gammap=par(29)
      gzero=par(30)
      ag=par(31)
      bg=par(32) 
       do m=1,natom
       tabv(m)=gcoor(m)
       enddo 
      ss=0d0
      sp=0d0
      do 31 i=1,natom
      if(i.eq.iatom)go to 31
      ria=r(i,iatom)
      terms=dexp(-deltass*ria)
      termp=dexp(-deltasp*ria)
      call fcnij(ds,gammas,gzero,ag,bg,r,tabv,fs,i,iatom)
      call fcnij(dp,gammap,gzero,ag,bg,r,tabv,fp,i,iatom)
      ss=ss+terms*fs
      sp=sp+termp*fp
   31 continue
      vs=vs+ss*deltacs
      vp=vp+sp*deltacp
      return
   40 deltacs=par(12) 
      deltass=par(13)
      deltacp=par(14)
      deltasp=par(15)
      b1=par(24)
      b2=par(25)
      b3=par(26)
      b4=par(27)
      ss=0d0
      sp=0d0
       do m=1,natom
       tabv(m)=cis(m)
       enddo 
      do 22 i=1,natom
      if(i.eq.iatom)go to 22
      ria=r(i,iatom)
      terms=dexp(-deltass*ria)
      termp=dexp(-deltasp*ria)
      call fbaij(b1,b2,b3,b4,r,tabv,fs,i,iatom)
      ss=ss+terms*fs
      sp=sp+termp
   22 continue
      vs=vs+ss*deltacs
      vp=vp+sp*deltacp
c      write(9,33)"vsips",s*delta,vsp
   33 format(A5,3d14.7)
c   30 continue
   50 return
      end


      subroutine envtab (r,natom,icase)
c  r --- two-dimensional array containing distances between all atoms
c  natom --- number of atoms
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension r(maxatom,maxatom)
      dimension par(npar)
      dimension csss(maxatom),csps(maxatom),cpps(maxatom),gcoor(maxatom)
      dimension cppp(maxatom),crep(maxatom),cis(maxatom),cip(maxatom) 
      common/param/par
      common/envir/csss,csps,cpps,cppp,crep,cis,cip,gcoor

      go to(20,30,40),icase
   20 return
   30 ag=par(31)
      cg=dexp(ag)
      bg=par(32)
      do 1 i=1,natom
      gcoor(i)=0d0
      do 2 j=1,natom
      if(i.eq.j) go to 2
      rij=r(i,j)
      if (rij.ge.bg) go to 2
      term=dexp(-ag/(1d0-(rij/bg)))
      gcoor(i)=gcoor(i)+term
   2  continue
      gcoor(i)=gcoor(i)*cg
   1  continue
      return
   40 continue

c
c  parameters for bond angle terms csss(i),csps(i),cis(i),cip(i)
c     beta2
c     beta3
c
      do 10 i=1,natom
      csss(i)=0d0
      csps(i)=0d0
c      cpps(i)=0d0
c      cppp(i)=0d0
c      crep(i)=0d0
      cis(i)=0d0
c      cip(i)=0d0
      do 9 j=1,natom
      if(i.eq.j) go to 9
      rij=r(i,j)
c
      beta2=par(17)
      beta3=par(18)
      csss(i)=csss(i)+dexp(-beta2*rij**beta3)
c
      beta2=par(21)
      beta3=par(22)
      csps(i)=csps(i)+dexp(-beta2*rij**beta3)
c
      beta2=par(25)
      beta3=par(26)
      cis(i)=cis(i)+dexp(-beta2*rij**beta3)
c
c
    9 continue
   10 continue
c      write(9,*)'csss'
c      write(9,15)(csss(i),i=1,natom)
c
c      write(9,*)'csps'
c      write(9,15)(csps(i),i=1,natom)
c
c      write(9,*)'cpps'
c      write(9,15)(cpps(i),i=1,natom)
c
c      write(9,*)'cppp'
c      write(9,15)(cppp(i),i=1,natom)
c
c      write(9,*)'crep'
c      write(9,15)(crep(i),i=1,natom)
c
c      write(9,*)'cis'
c      write(9,15)(cis(i),i=1,natom)
c      write(9,*)'cip'
c      write(9,15)(cip(i),i=1,natom)
   15 format(5d12.4)

      return
      end
c
c
      subroutine pot_al (x,y,z,r,hamil,repul,natom,i,j,icase)
c  x,y,z --- one-dimensional arrays of the Cartesian coordinates 
c            of the system.
c  r --- two-dimensional array containing distances between all atoms
c   hamil ---4x4 dimensional submatrix for the interaction between
c          ith and jth atoms
c  repul --- repulsion energy between ith and jth atoms
c  natom --- number of atoms in the system
c  i,j   --- indices of the two considered atoms

c
      implicit real*8(a-h,o-z)
      save
      include "maxdim.h"
      dimension x(maxatom),y(maxatom),z(maxatom),r(maxatom,maxatom)
      dimension par(npar),tabv(maxatom)
      dimension hamil(norbit,norbit)
      dimension csss(maxatom),csps(maxatom),cpps(maxatom),gcoor(maxatom)
      dimension cppp(maxatom),crep(maxatom),cis(maxatom),cip(maxatom) 
      common/envir/csss,csps,cpps,cppp,crep,cis,cip,gcoor
      common/param/par
c
      rij=r(i,j)
c      write(9,*)'par'
c      write(9,201)(par(m),m=1,npar)
c      write(9,*)'r'
c      write(9,201)rij

c   
c Vsss potential parameters
      eta=par(1)
      zeta=par(2)
      call potsss(rij,eta,zeta,sss,dsss)
c   
c Vsps potential parameters
      eta=par(3)
      zeta=par(4)
      call potsps(rij,eta,zeta,sps,dsps)
c   
c Vpps potential parameters
      eta=par(5)
      zeta=par(6)
      call potpps(rij,eta,zeta,pps,dpps)
c   
c Vppp potential parameters
      eta=par(7)
      zeta=par(8)
      call potppp(rij,eta,zeta,ppp,dppp)
c      f1=1d0
c      do 14 m=1,natom
c   14 tabv(m)=cppp(m)
c      if(beta1.ne.0d0) call fbaij(beta1,beta2,beta3,beta4,r,tabv,f1,i,j)
c      write(9,*)'ppp'
c      write(9,201)ppp,f1
c      write(9,*)i,j
c      ppp=ppp*f1
    4 continue
c   
c Vrep potential parameters
      aa=par(9)
      enn=par(10)
      bb=par(11)
      call potrep(rij,aa,enn,bb,repul,drepul)
c
c
      go to (20,30,40,50,50,50),icase
   20 b1=par(16)
      b2=par(17)
      b3=par(18)
      call fsij(b1,b2,b3,natom,r,f1,i,j)
      sss=sss*f1
      b1=par(19)
      b2=par(20)
      b3=par(21)
      call fsij(b1,b2,b3,natom,r,f1,i,j)
      sps=sps*f1
      go to 50
   30 do 31 m=1,natom
   31 tabv(m)=gcoor(m)
      gzero=par(30)
      ag=par(31)
      bg=par(32) 
      d=par(16)
      gamma=par(17)      
      call fcnij(d,gamma,gzero,ag,bg,r,tabv,f1,i,j)
      sss=sss*f1       
      d=par(18)
      gamma=par(19)      
      call fcnij(d,gamma,gzero,ag,bg,r,tabv,f1,i,j)
      sps=sps*f1       
      d=par(20)
      gamma=par(21)      
      call fcnij(d,gamma,gzero,ag,bg,r,tabv,f1,i,j)
      pps=pps*f1       
      d=par(22)
      gamma=par(23)      
      call fcnij(d,gamma,gzero,ag,bg,r,tabv,f1,i,j)
      ppp=ppp*f1       
      d=par(24)
      gamma=par(25)      
      call fcnij(d,gamma,gzero,ag,bg,r,tabv,f1,i,j)
      repul=repul*f1       
      go to 50
   40 beta1= par(16)
      beta2=par(17)
      beta3=par(18)
      beta4=par(19)
      do  m=1,natom
      tabv(m)=csss(m)
      enddo
      call fbaij(beta1,beta2,beta3,beta4,r,tabv,f1,i,j)
      sss=sss*f1
      beta1= par(20)
      beta2=par(21)
      beta3=par(22)
      beta4=par(23)
      do  m=1,natom
      tabv(m)=csps(m)
      enddo
      call fbaij(beta1,beta2,beta3,beta4,r,tabv,f1,i,j)
      sps=sps*f1
c
c      write(9,*)
c      write(9,201)sss,sps,pss,pps,ppp
c      write(9,*)
c Project the two-center integral along the basis set.
c
   50 continue
      rij=r(i,j)
      el=(x(i)-x(j))/rij
      em=(y(i)-y(j))/rij
      en=(z(i)-z(j))/rij
      hamil(1,1)=sss

      t=el*sps       !s-px
      hamil(1,2)=t
      hamil(2,1)=-t
      t=em*sps       !s-py
      hamil(1,3)=t
      hamil(3,1)=-t
      t=en*sps
      hamil(1,4)=t
      hamil(4,1)=-t
      t1=pps-ppp
      hamil(2,2)=el*el*t1+ppp
      hamil(3,3)=em*em*t1+ppp
      hamil(4,4)=en*en*t1+ppp
      t=t1*el*em  !px-py
      hamil(2,3)=t
      hamil(3,2)=t
      t=t1*el*en  !px-pz
      hamil(2,4)=t
      hamil(4,2)=t
      t=t1*em*en  !py-px
      hamil(3,4)=t
      hamil(4,3)=t
c
c      do 200 m=1,4
c  200 write(9,201)(hamil(m,m1),m1=1,4)
  201 format(8d14.6)
c
      return
      end
c
c
      subroutine potsss (r,eta,beta,V,dV)
c  r ---  distance between two atoms
c  eta, beta --- TB parameters modeling the potential function
c  V  --- the value of the interaction
c  dV --- dummy variable (is to be derivative in the future)
      implicit real*8(a-h,o-z) 
      save
      DATA c2,c3,c4,c5,c6/.46666666666666667D+00,.13366333333336333D+00,
     *.266666666666666667D-01,.4444444444444664D-02,.6349206349206D-03/
c      c2=7./15.,  c3=2./15., c4=2./75., c5=1./225., c6=1./1575.
c
      c0=1d0
      c1=1d0
      p1=beta*r
      p2=p1*p1
      p3=p2*p1
      p4=p3*p1
      p5=p4*p1
      p6=p5*p1
      ep=dexp(-p1)
      w=c0+p1+p2*c2+p3*c3+p4*c4+p5*c5+p6*c6
      V=-w*ep*eta
c      write(9,*)
c      write(9,80)"sss  ", v,r,eta,beta
c      write(9,80)"inne ", w,ep
  80  format(A5,8d14.7)
      return
      end

      subroutine potsps(r,eta,beta,V,dV)
c  r ---  distance between two atoms
c  eta, beta --- TB parameters modeling the potential function
c  V  --- the value of the interaction
c  dV --- dummy variable (is to be derivative in the future)
      implicit real*8(a-h,o-z) 
      save
      DATA c2,c3,c4,c5,c6/.19245008972988D+00,.92376043070340D-01,
     *.28226013160382D-01,.62317171912531D-02,.1099714798456D-02/
c          c2=1d0/c, c3=12./25./c, c4=11./75./c, c5=17./525./c,
c          c6=1d0/175./c, c=dsqrt(27)
c
      p1=beta*r
      p2=p1*p1
      p3=p2*p1
      p4=p3*p1
      p5=p4*p1
      p6=p5*p1
      ep=dexp(-p1)
      w=(p1+p2)*c2+p3*c3+p4*c4+p5*c5+p6*c6
      V=w*ep*eta
      return
      end

      subroutine potpps (r,eta,beta,V,dV)
c  r ---  distance between two atoms
c  eta, beta --- TB parameters modeling the potential function
c  V  --- the value of the interaction
c  dV --- dummy variable (is to be derivative in the future)
      implicit real*8(a-h,o-z) 
      save
      DATA c2,c3,c4,c5,c6/.36000000000000D+00,.26666666666667D-01,
     *.21587301587302D-01,.82539682539683D-02,.1904761904762D-02/
c  c1=-1d0, c2=-9./25., c3=-2./75., c4=34./1575., c5=13./1575. c6=1d0/525
c
      p1=beta*r
      p2=p1*p1
      p3=p2*p1
      p4=p3*p1
      p5=p4*p1
      p6=p5*p1
      ep=dexp(-p1)
      w=-1d0-p1-p2*c2-p3*c3+p4*c4+p5*c5+p6*c6
      V=w*ep*eta
      return
      end

      subroutine potppp (r,eta,beta,V,dV)
c  r ---  distance between two atoms
c  eta, beta --- TB parameters modeling the potential function
c  V  --- the value of the interaction
c  dV --- dummy variable (is to be derivative in the future)
      implicit real*8(a-h,o-z) 
      save
      DATA c2,c3,c4,c5/.45333333333333D+00,.12000000000000D+00,
     *.19682539682540D-01,.1904761904762D-02/
c c0=1d0, c1=1d0, c2=34./75., c3=3./25., c4=31./1575, c5=1./525
c
      p1=beta*r
      p2=p1*p1
      p3=p2*p1
      p4=p3*p1
      p5=p4*p1
      p6=p5*p1
      ep=dexp(-p1)
      w=1d0+p1+p2*c2+p3*c3+p4*c4+p5*c5
      V=-w*ep*eta
      return
      end
 

c
      subroutine potrep(r,aa,enn,alp,u1,dt)
c  r ---  distance between two atoms
c  aa, enn,alp  --- TB parameters modeling the potential function
c  u1  --- the value of the interaction
c  dt --- dummy variable (is to be derivative in the future)
      implicit real*8(a-h,o-z)
      save
      rel=1d0/r
      u1=aa*(rel**enn)*dexp(-alp*r)
   2  format(f6.2,3d14.6)
      return
      end
c
c
c Arrange all the valence electrons to the necessary molecular orbitals 
c so that the potential energy of the system is minimized.
c
      subroutine filloc(ndim,nElect,eva,etb,nocc)
c
      implicit real*8(a-h,o-z)
      include "maxdim.h"
c
      dimension eva(maxdim),vsip(2)
      dimension nocc(maxdim)
      common /parsys/ vsip,u,cem,unite,unitd
c
c nElect --- number of valence electrons in the system.
c ndim --- dimension of the Hamiltonian matrix.
c
c eva  --- an array of the eigenvalues of the molecular orbitals for the
c           system
c etb  --- the valence bond energy of the system
c nocc --- an array with the set of occupation numbers of electrons for
c           the MOs.

c
c Initialize occupation number set
c
      do 10 i=1,ndim
         nocc(i)=0
   10 continue
c
      if (nElect .gt. 2*ndim) stop "There is not enough MOs"
c
c Arrange electrons simply from lowest orbitals to highest orbitals
c
      nbottom=nElect/2
      if (nElect .eq. 1) then
        nocc(1)=1
        etb=eva(1)
        return
      else
        do 20 i=1, nElect/2
          nocc(i)=2
   20   continue
        if (2*(nElect/2) .ne. nElect) then
          nocc(i)=1
          ntop=nbottom+2
        else
          ntop=nbottom+1
        endif
      endif
c
c Rearrange the occupation according to penalty energy to obtain a set of 
c occupancy to give minimum energy.
c At first, the orbitals are occupied from lowest to higher orbital with two
c electrons if there are enough electrons, which is called the starting point. 
c The rearrangement of the the molecular orbitals begins here.  The highest 
c doubly occupied orbital is called nbottom, and the lowest unoccupied MO is
c called ntop.  If the energy difference between ntop and nbottom is smaller
c than the penalty energy (u or 0.80 eV in this case), the electron from 
c nbottom jump to ntop.  Otherwise, the rearrangement will stop. 
c
      if (nbottom .eq. 0) goto 102
      if (ntop .gt. ndim) goto 102
c
 101  if (eva(nbottom)+u .gt. eva(ntop)) then 
c
c If the energy of the top orbital is higher by u than that of the bottom 
c orbital, the bottom becomes top and the number of the bottom orbital is 
c decreased by 1.
c
        nocc(ntop)=1
        ntop=ntop+1
        nocc(nbottom)=1
        nbottom=nbottom-1
c
        if (ntop .gt. ndim) goto 102
        if (nbottom .eq. 0) goto 102
c
        goto 101
c
      endif
c
c Calculate the energy of resulting structure.
c
 102  continue
      etb=0.0
      do k=1,ndim
        if (nocc(k).eq.2) then
          etb=etb+u+eva(k)*2
        else
          etb=etb+eva(k)*nocc(k)
        endif
      enddo
c
      return
      end
c
c
c######################################################################
c
      subroutine case(icase)
c  icase --- parameter teling which model is to be used
c          1--MBTB-S
c          2--MBTB-CN
c          3--MBTB-BA
c          4--OWH
c          5--EWH
c          6--WH
      implicit real*8 (a-h,o-z)
c      save
      include "maxdim.h"      
      dimension parent1(npar1),parent2(npar2),parent3(npar3)
     *,parent4(npar4),parent5(npar4),parent6(npar4),parent(npar),vsip(2)
      common /param/ parent
      common /parsys/ vsip,upenalty,cem,unite,unitd
c
      DATA (parent1(i),i=1,24)/  .13962D+00,  .88700D+00,  .85619D-01,
     *  .94438D+00,  .64375D-01,  .13601D+01,  .43771D-01,  .13657D+01,
     *  .20932D+03,  .38349D+00,  .20615D+01,  .22855D+00,  .17966D+01,
     *  .82999D-01,  .60177D+00,  .37227D-01,  .92029D+00,  .74127D+00,
     *  .23998D+01,  .99128D+00,  .22524D+01,  .61485D+00,  .11767D+01,
     *  .13692D+01/
c
      DATA (parent2(i),i=1,32)/  .22562D+00,  .86924D+00,  .57356D-01,
     *  .93174D+00,  .74159D-01,  .13420D+01,  .27552D-01,  .11172D+01,
     *  .11960D+03,  .41247D+00,  .18539D+01,  .11087D+00,  .19931D+01,
     *  .82532D-01,  .61645D+00,  .12900D+00,  .41719D+01,  .20609D-01,
     *  .31250D+01,  .96560D-01,  .42367D+01,  .17679D+00,  .41112D+01,
     *  .13771D+00,  .21194D+01,  .90019D+00,  .26656D+01,  .89967D+00,
     *  .18384D+01,  .12603D+02,  .16917D+01,  .15108D+02/
c
      DATA (parent3(i),i=1,27)/      .97120D-01, .75305D+00, .10473D+00,
     *   .11378D+01,   .57150D-01,   .12311D+01, .40020D-01, .13154D+01,
     *   .17944D+03,   .22539D+00,   .20578D+01, .16685D+00, .12383D+01,
     *   .14256D+00,   .72432D+00,   .28270D+01, .10484D+01, .19357D+01,
     *   .36467D+01,   .18839D+00,   .15270D+01, .16355D+01, .24757D+01,
     *   .95855D+00,   .18893D+01,   .18120D+01, .15924D+01/
c
      DATA (parent4(i),i=1,11)/ .15308D+00,  .10038D+01,  .95139D-01,
     *  .96784D+00,  .94412D-01,  .20513D+01,  .13053D+01,  .33225D+01,
     *  .11987D+03,  .95543D-01,  .18773D+01/

c
      DATA (parent5(i),i=1,11)/  .14604D+00,  .13724D+01,  .15809D+00,
     *  .13638D+01,  .70843D-01,  .13552D+01,  .21941D-01,  .13552D+01,
     *  .25117D+02,  .65780D-01,  .13838D+01/
c
      DATA (parent6(i),i=1,11)/  .39280D+00,  .13724D+01,  .39280D+00,
     *  .13638D+01,  .39280D+00,  .13552D+01,  .39280D+00,  .13552D+01,
     *  .42747D+02,  .29360D-01,  .15459D+01/
c
c   valence state ionization potentials in eV 
c
c      data (vsip(i),i=1,2) /-10.62,-5.986/  Al after 11/13/02 
c      cem=3.43 eV
c      upenalty=0.07 eV
c      unite=0.03674901d0 eV -> a.u.
c      unitd=1.889727d0    A -> a.u.
c   valence state ionization potentials in a.u.
c
c      data (vsip(i),i=1,2)  /-0.3902744862d0,-0.21997957386d0/
c      data cem,upenalty, unite, unitd /-0.1260491043d0,0.0025724307d0
c     *,0.03674901d0,1.889727d0/
c      vsip(1)=-0.3902744862d0
c      vsip(2)=-0.21997957386d0
c      cem=-0.1260491043d0
c      upenalty=0.0025724307d0
c      unite=0.03674901d0
c      unitd=1.889727d0
      go to (10,20,30,40,50,60),icase
   10 do 1 i=1,npar1
    1 parent(i)=parent1(i)
c
      go to 70
   20 do 2 i=1,npar2
    2 parent(i)=parent2(i)
c
      go to 70
   30 do 3 i=1,npar3
    3 parent(i)=parent3(i)
c
      go to 70
   40 do 4 i=1,npar4
    4 parent(i)=parent4(i)
c
      go to 70
   50 do 5 i=1,npar4
    5 parent(i)=parent5(i)
c
      go to 70
   60 do 6 i=1,npar4
    6 parent(i)=parent6(i)
      parent(1)=-parent6(1)*vsip(1)
      parent(3)=-parent6(1)*(vsip(1)+vsip(2))/2
      parent(5)=-parent6(1)*vsip(2)
      parent(7)=-parent6(1)*vsip(2)
   70 continue
      return
      end
cc
c
      BLOCK DATA set1
      implicit real*8 (a-h,o-z)
c      save
c      include "maxdim.h"      
      dimensionvsip(2)
      common /parsys/ vsip,upenalty,cem,unite,unitd
      data (vsip(i),i=1,2)  /-0.3902744862d0,-0.21997957386d0/
c   cem=3.43eV
c   upenalty=0.07eV
c  in data we have a.u.
      data cem,upenalty, unite, unitd /-0.1260491043d0,0.0025724307d0
     *,0.03674901d0,1.889727d0/
      end

